(module ecore-main mzscheme
  
  (require (lib "foreign.ss") 
           "ecore-common.ss")

  (define libecore (ffi-lib "libecore"))
  
  (define-efl-exn-raiser ecore)
  (define-efl-ffi-definer ecore)
  
  (define _EcoreFdHandlerFlags
    (_bitmask '(ECORE-FD-READ = 1
                ECORE-FD-WRITE = 2
                ECORE-FD-ERROR = 4)))
  
  (define _EcoreEventType
    (_enum '(ECORE-EVENT-NONE = 0
             ECORE-EVENT-EXE-EXIT = 1
             ECORE-EVENT-SIGNAL-USER = 2
             ECORE-EVENT-SIGNAL-HUP = 3
             ECORE-EVENT-SIGNAL-EXIT = 4
             ECORE-EVENT-SIGNAL-POWER = 5
             ECORE-EVENT-SIGNAL-REALTIME = 6)))
  
  (define _siginfo-t _sint) ; WRONG!
  
  (define-cpointer-type _EcoreExe)
  (define-cpointer-type _EcoreTimer)
  (define-cpointer-type _EcoreIdler)
  (define-cpointer-type _EcoreIdleEnterer)
  (define-cpointer-type _EcoreIdleExiter)
  (define-cpointer-type _EcoreFdHandler)
  (define-cpointer-type _EcoreEventHandler)
  (define-cpointer-type _EcoreEventFilter)
  (define-cpointer-type _EcoreEvent)
  (define-cpointer-type _EcoreAnimator)
  
  (define-cstruct* _EcoreEventExeExit
    ((pid _pid-t) (exit-code _sint) (exe _EcoreExe) (exit-signal _sint)
     (exited/signaled _uint8) (ext-data _pointer) (data _siginfo-t)))
  (define-cstruct* _EcoreEventSignalUser
    ((number _sint) (ext-data _pointer) (data _siginfo-t)))
  (define-cstruct* _EcoreEventSignalHup
    ((ext-data _pointer) (data _siginfo-t)))
  (define-cstruct* _EcoreEventSignalExit
    ((interrupt/quit/terminate _sint) (ext-data _pointer) (data _siginfo-t)))
  (define-cstruct* _EcoreEventSignalPower
    ((ext-data _pointer) (data _siginfo-t)))
  (define-cstruct* _EcoreEventSignalRealtime
    ((num _sint) (data _siginfo-t)))
  
  (define init-count 0)
  
  ;;Functions
  (defecore* ecore-init :
    (post: (set! init-count o_init-count))
    -> (o_init-count : _sint))
  (defecore* ecore-shutdown :
    (post: (set! init-count o_init-count))
    -> (o_init-count : _sint))
  
  (defecore* ecore-app-args-set :
    (_sint = (add1 (vector-length i_argv)))
    (i_argv : (_vector i _string))
    -> _void)

  (defecore* ecore-app-args-get :
    (o_argc : (_ptr o _sint))
    (o_argv : (_ptr o (_vector o _string o_argc)))
    -> _void
    -> o_argv)
  
  (defecore* ecore-app-restart : -> _void)
  
  (defecore* ecore-event-handler-add :
    _sint (_fun _scheme _sint _pointer -> _sint) _scheme
    -> _EcoreEventHandler)
  (defecore* ecore-event-handler-del :
    _sint (_fun _scheme _sint _pointer -> _sint)
    -> _scheme)
  
  (defecore* ecore-event-add :
    _EcoreEventType _pointer (_fun _scheme _pointer -> _cbvoid) _scheme
    -> _EcoreEvent)
  (defecore* ecore-event-del : _EcoreEvent -> _scheme)
  
  (defecore* ecore-event-type-new : -> _sint)
  
  (defecore* ecore-event-filter-add :
    (_fun _scheme -> _pointer)
    (_fun _scheme _pointer _EcoreEventType _pointer -> _sint)
    (_fun _scheme _pointer -> _cbvoid)
    _scheme
    -> _EcoreEventFilter)
  (defecore* ecore-event-filter-del : _EcoreEventFilter -> _scheme)
  
  (defecore* ecore-event-current-type-get : -> _sint)
  (defecore* ecore-event-current-event-get : -> _pointer)
  
  (define (ecore-exe-finalize ee)
    (if (> init-count 0) (ecore-exe-free ee)))
  (defecore ecore-exe-free : _EcoreExe -> _pointer)
  (defecore* ecore-exe-run :
    (post: (if o_exe (register-finalizer o_exe ecore-exe-finalize)))
    _string _pointer
    -> (o_exe : _EcoreExe/null))
  
  (defecore* ecore-exe-pid-get : _EcoreExe -> _pid-t)
  (defecore* ecore-exe-data-get : _EcoreExe -> _pointer)
  (defecore* ecore-exe-pause : _EcoreExe -> _void)
  (defecore* ecore-exe-continue : _EcoreExe -> _void)
  (defecore* ecore-exe-terminate : _EcoreExe -> _void)
  (defecore* ecore-exe-kill : _EcoreExe -> _void)
  (defecore* ecore-exe-signal : _EcoreExe _sint -> _void)
  (defecore* ecore-exe-hup : _EcoreExe -> _void)
  
  (defecore* ecore-idler-add :
    (_fun _scheme -> _sint) _scheme
    -> _EcoreIdler)
  (defecore* ecore-idler-del : _EcoreIdler -> _scheme)
  
  (defecore* ecore-idle-enterer-add :
    (_fun _scheme -> _sint) _scheme
    -> _EcoreIdleEnterer)
  (defecore* ecore-idle-enterer-del : _EcoreIdleEnterer -> _scheme)
  
  (defecore* ecore-idle-exiter-add :
    (_fun _scheme -> _sint) _scheme
    -> _EcoreIdleExiter)
  (defecore* ecore-idle-exiter-del : _EcoreIdleExiter -> _scheme)
  
  (defecore* ecore-main-loop-iterate : -> _void)
  (defecore* ecore-main-loop-begin : -> _void)
  (defecore* ecore-main-loop-quit : -> _void)
  
  (defecore* ecore-main-fd-handler-add :
    _sint (_fun _scheme _EcoreFdHandler -> _sint)
    _scheme (_fun _scheme _EcoreFdHandler -> _sint)
    _scheme
    -> _EcoreFdHandler)
  (defecore* ecore-main-fd-handler-del : _EcoreFdHandler -> _void)
  (defecore* ecore-main-fd-handler-fd-get : _EcoreFdHandler -> _sint)
  
  (defecore* ecore-main-fd-handler-active-get :
    _EcoreFdHandler _EcoreFdHandlerFlags
    -> _bool)
  (defecore* ecore-main-fd-handler-active-set :
    _EcoreFdHandler _EcoreFdHandlerFlags
    -> _void)
  
  (defecore* ecore-time-get : -> _double)
  (defecore* ecore-timer-add :
    _double (_fun _scheme -> _sint) _scheme
    -> _EcoreTimer)
  (defecore* ecore-timer-del : _EcoreTimer -> _scheme)
  (defecore* ecore-timer-interval-set : _EcoreTimer _double -> _void)
  
  (defecore* ecore-animator-add :
    (_fun _scheme -> _sint) _scheme
    -> _EcoreAnimator)
  (defecore* ecore-animator-del : _EcoreAnimator -> _scheme)
  (defecore* ecore-animator-frametime-set : _double -> _void)
  (defecore* ecore-animator-frametime-get : -> _double))
