(module edb mzscheme
  
  (require (lib "foreign.ss") 
           "efl-common.ss")
  
  (define libedb (ffi-lib "libedb"))
  
  (provide (struct edb-error ()) (struct edb-error:I/O ()))
  (define-struct (edb-error efl-error) ())
  (define-struct (edb-error:I/O edb-error) ())
  
  (define-efl-exn-raiser edb)
  (define-efl-ffi-definer edb)
  
  (define-cpointer-type _EDBFile)

  (define _EDBFileMode
    (_bitmask '(O-ACCMODE = #o000003
                O-RDONLY = #o000000
                O-WRONLY = #o000001
                O-RDWR = #o000002
                O-CREAT = #o000100
                O-EXCL = #o000200
                O-NOCTTY = #o000400
                O-TRUNC = #o001000
                O-APPEND = #o002000
                O-NONBLOCK = #o004000
                O-NDELAY = #o004000
                O-SYNC = #o010000
                O-FSYNC = #o010000
                O-ASYNC = #o020000
                O-DIRECT = #o040000
                O-DIRECTORY = #o200000
                O-NOFOLLOW = #o400000
                EDB-FILE-MODE-READ = #o000000
                EDB-FILE-MODE-WRITE = #o000001
                EDB-FILE-MODE-READ-WRITE = #o000002
                EDB-FILE-MODE-WRITE/CREATE = #o000101
                EDB-FILE-MODE-READ-WRITE/CREATE = #o000102)))
  
  ;; Functions
  (defedb* e-db-open :
    (post: (unless o_e-db-file
             (edb-raise-exn I/O : "Failed to open ~a." i_file)))
    (i_file : _file)
    -> (o_e-db-file : _EDBFile/null))
  (defedb* e-db-open-mode :
    (post: (unless o_e-db-file
             (edb-raise-exn I/O : "Failed to open ~a." i_file)))
    (i_file : _file) (i_flags : _EDBFileMode)
    -> (o_e-db-file : _EDBFile/null))
  
  (defedb* e-db-property-set :
    _EDBFile _string _string -> _void)
  (defedb* e-db-property-get :
    _EDBFile _string -> _string)
  
  (defedb* e-db-set-type :
    _EDBFile _string -> _void)
  (defedb* e-db-is-type? :
    _EDBFile _string -> _bool)
  
  (defedb* e-db-open-read :
    (post: (unless o_e-db-file
             (edb-raise-exn I/O : "Failed to open ~a." i_file)))
    (i_file : _file)
    -> (o_e-db-file : _EDBFile))
  (defedb* e-db-close :
    _EDBFile -> _void)
  
  (defedb* e-db-data-get :
    (post: (if (not o_data)
               (edb-raise-exn I/O : "Failed to read data from '~a'." i_key)
               (register-finalizer o_data free)))
    _EDBFile (i_key : _string) (o_size : (_ptr o _sint))
    -> (o_data : (_byte-vector o_size)))
  (defedb* e-db-data-set :
    _EDBFile _string (i_data : _bytes) (_sint = (bytes-length i_data))
    -> _void)
  (defedb* e-db-data-del :
    _EDBFile _string -> _void)
  
  (defedb* e-db-usage :
    -> _sint)
  (defedb* e-db-flush :
    -> _void)
  (defedb* e-db-runtime-flush :
    -> _sint)
  
  (defedb* e-db-int-set :
    _EDBFile _string _sint -> _void)
  (defedb* e-db-int-get :
    (post: (unless o_success
             (edb-raise-exn I/O : "Failed to read from '~a'." i_key)))
    _EDBFile (i_key : _string) (o_val : (_ptr o _sint))
    -> (o_success : _bool)
    -> o_val)
  
  (defedb* e-db-float-set :
    _EDBFile _string _float -> _void)
  (defedb* e-db-float-get :
    (post: (unless o_success
             (edb-raise-exn I/O : "Failed to read from '~a'." i_key)))
    _EDBFile (i_key : _string) (o_val : (_ptr o _float))
    -> (o_success : _bool)
    -> o_val)
  
  (defedb* e-db-str-set :
    _EDBFile _string _string -> _void)
  (defedb* e-db-str-get :
    (post: (unless o_str
             (edb-raise-exn I/O : "Failed to read from '~a'." i_key)))
    _EDBFile (i_key : _string)
    -> (o_str : _string/utf-8/free))
  
  (defedb* e-db-type-set :
    _EDBFile _string _string -> _void)
  (defedb* e-db-type-get :
    (post: (unless o_str
             (edb-raise-exn I/O : "Failed to read from '~a'." i_key)))
    _EDBFile (i_key : _string)
    -> (o_str : _string/utf-8/free))
  
  (defedb* e-db-dump-multi-field :
    (i_file : _file) (i_file2 : _file) (o_num : (_ptr o _sint))
    -> (o_ret : (_list/free _string/utf-8/free o_num)))
  
  (defedb* e-db-dump-key-list :
    _file (o_num : (_ptr o _sint))
    -> (o_ret : (_list/free _string/utf-8/free o_num)))
  
  (defedb* e-db-match-keys :
    _EDBFile _string (o_num : (_ptr o _sint))
    -> (o_ret : (_list/free _string/utf-8/free o_num)))

  (provide call-with-e-db-file)
  (define (call-with-e-db-file filename mode proc)
    (define e-db-file #f)
    (dynamic-wind
     (lambda () (set! e-db-file (e-db-open-mode filename mode)))
     (lambda () (proc e-db-file))
     (lambda () (e-db-close e-db-file)))))