#!/bin/sh
#|
exec mzscheme -qr "$0" ${1+"$@"}
|#

(require (lib "foreign.ss"))
(define libpthread (ffi-lib "/lib/libpthread.so.0"))
(require "ecore-main.ss" "evas.ss" "ecore-evas.ss" "edje.ss")
;(require (lib "calltrace.ss" "trace"))

(define *stop* #f)

(define *ee* #f)
(define *evas* #f)

(define *o-bg* #f)
(define *o-edje* #f)

(define (signal-cb data o sig src)
  (printf "EMIT: \"~a\" \"~a\"\n" sig src))

(define (resize-cb ee)
  (let* ([ret (evas-output-viewport-get *evas*)]
         [w (caddr ret)]
         [h (cadddr ret)])
    (evas-object-resize *o-bg* w h)
    (evas-object-resize *o-edje* w h)))

(define (delete-cb ee)
  ;(ecore-main-loop-quit)
  (set! *stop* #t))

(define (key-cb data e o ei)
  (printf "KEY: ~a\n" (EvasEventKeyDown-keyname ei))
  (case (string->symbol (EvasEventKeyDown-keyname ei))
    ((Escape) (set! *stop* #t));(ecore-main-loop-quit))
    ((space) (edje-object-message-send *o-edje* 'EDJE-MESSAGE-NONE 77 #f))
    ((|1|) (edje-object-message-send *o-edje* 'EDJE-MESSAGE-STRING 101
                                     (make-EdjeMessageString "A Test String")))
    ((|2|) (edje-object-message-send *o-edje* 'EDJE-MESSAGE-INT 102
                                     (make-EdjeMessageInt 42)))
    ((|3|) (edje-object-message-send *o-edje* 'EDJE-MESSAGE-FLOAT 103
                                     (make-EdjeMessageFloat 3.14159)))
    ((|4|) void)
    ((|5|) void)
    ((|6|) void)
    ((|7|) (edje-object-message-send *o-edje* 'EDJE-MESSAGE-STRING-INT 107
                                     (make-EdjeMessageStringInt
                                      "A Test String"
                                      42)))
    ((|8|) (edje-object-message-send *o-edje* 'EDJE-MESSAGE-STRING-FLOAT 108
                                     (make-EdjeMessageStringFloat
                                      "A Test String and Float"
                                      777.777)))
    ((|9|) void)
    ((|0|) void)))

(define (message-cb data obj type id msg)
  (printf "MESSAGE FROM OBJ, TYPE ~a, ID ~a\n" type id)
  (case type
    ((EDJE-MESSAGE-NONE) void)
    ((EDJE-MESSAGE-STRING) (printf "  STRING: \"~a\"\n"
                                   (EdjeMessageString-str msg)))
    ((EDJE-MESSAGE-INT) (printf "  INT: ~a\n"
                                (EdjeMessageInt-val msg)))
    ((EDJE-MESSAGE-FLOAT) (printf "  FLOAT: ~a\n"
                                  (EdjeMessageFloat-val msg)))
    ((EDJE-MESSAGE-STRING-SET) void)
    ((EDJE-MESSAGE-INT-SET) void)
    ((EDJE-MESSAGE-FLOAT-SET) void)
    ((EDJE-MESSAGE-STRING-INT) void)
    ((EDJE-MESSAGE-STRING-FLOAT) void)
    ((EDJE-MESSAGE-STRING-INT-SET) void)
    ((EDJE-MESSAGE-STRING-FLOAT-SET) void)))

(define (main)
  (define o #f)

  (when (not (= (vector-length argv) 2))
    (display "Usage: edje_test edje_file.edj part_to_load\n")
    (exit))
  
  (ecore-init)
  (ecore-evas-init)
  (edje-init)

  (set! *ee* (ecore-evas-software-x11-new #f 0 0 0 240 320))
  (set! *evas* (ecore-evas-get *ee*))
  (evas-image-cache-set *evas* (* 8192 1024))
  (evas-font-cache-set *evas* (* 512 1024))
  
  (set! o (evas-object-rectangle-add *evas*))
  (evas-object-move o 0 0)
  (evas-object-resize o 240 320)
  (evas-object-color-set o 255 255 255 255)
  (evas-object-event-callback-add o 'EVAS-CALLBACK-KEY-DOWN key-cb #f)
  (evas-object-focus-set o #t)
  (evas-object-show o)
  (set! *o-bg* o)
  
  (set! o (edje-object-add *evas*))
  (edje-object-signal-callback-add o "*" "*" signal-cb #f)
  (edje-object-message-handler-set o message-cb #f)
  (edje-object-file-set o (vector-ref argv 0) (vector-ref argv 1))
  (evas-object-move o 0 0)
  (evas-object-resize o 240 320)
  (evas-object-show o)
  (set! *o-edje* o)
  
  (ecore-evas-callback-resize-set *ee* resize-cb)
  (ecore-evas-callback-delete-request-set *ee* delete-cb)
  (ecore-evas-show *ee*)
 
  (let loop ()
    (ecore-main-loop-iterate)
    (unless *stop* (loop)))

  (edje-shutdown)
  (ecore-evas-shutdown)
  (ecore-shutdown))

(main)
#|
(define t (thread main))

(sleep 5)
(define line (evas-object-line-add *evas*))
(evas-object-color-set line 255 100 100 255)
(evas-object-show line)
  
(let run ()
  (let loop1 ((x1 100) (y1 100) (x2 200) (y2 200))
    (sleep .01)
    (evas-object-line-xy-set line x1 y1 x2 y2)
    (unless (= 100 x2) (loop1 (+ x1 2) (add1 y1)
			      (- x2 2) (sub1 y2))))
  (let loop2 ((x1 200) (y1 150) (x2 100) (y2 150))
    (sleep .01)
    (evas-object-line-xy-set line x1 y1 x2 y2)
    (unless (= 100 x1) (loop2 (- x1 2) (add1 y1)
			      (+ x2 2) (sub1 y2))))
  (let loop3 ((x1 100) (y1 200) (x2 200) (y2 100))
    (sleep .01)
    (evas-object-line-xy-set line x1 y1 x2 y2)
    (unless (= 100 x2) (loop3 (+ x1 2) (sub1 y1)
			      (- x2 2) (add1 y2))))
  (let loop4 ((x1 200) (y1 150) (x2 100) (y2 150))
    (sleep .01)
    (evas-object-line-xy-set line x1 y1 x2 y2)
    (unless (= 100 x1) (loop4 (- x1 2) (sub1 y1)
			      (+ x2 2) (add1 y2))))

  (unless *stop* (run)))|#
