(module edje mzscheme
  
  (require (lib "foreign.ss")
           "efl-common.ss"
           "evas-types.ss")
  (unsafe!)
  (define libedje (ffi-lib "libedje"))
  
  (define-efl-ffi-definer edje)
  
  (define _EdjeMessageType
    (_enum '(EDJE-MESSAGE-NONE = 0
             EDJE-MESSAGE-SIGNAL = 1
             EDJE-MESSAGE-STRING = 2
             EDJE-MESSAGE-INT = 3
             EDJE-MESSAGE-FLOAT = 4
             EDJE-MESSAGE-STRING-SET = 5
             EDJE-MESSAGE-INT-SET = 6
             EDJE-MESSAGE-FLOAT-SET = 7
             EDJE-MESSAGE-STRING-INT = 8
             EDJE-MESSAGE-STRING-FLOAT = 9
             EDJE-MESSAGE-INT-SET = 10
             EDJE-MESSAGE-STRING-FLOAT-SET = 11)))
  
  (define _EdjeDragDir
    (_enum '(EDJE-DRAG-DIR-NONE = 0
             EDJE-DRAG-DIR-X = 1
             EDJE-DRAG-DIR-Y = 2
             EDJE-DRAG-DIR-XY = 3)))
  
  (define _EdjeLoadError
    (_enum '(EDJE-LOAD-ERROR-NONE = 0
             EDJE-LOAD-ERROR-GENERIC = 1
             EDJE-LOAD-ERROR-DOES-NOT-EXIST = 2
             EDJE-LOAD-ERROR-PERMISION-DENIED = 3
             EDJE-LOAD-ERROR-RESOURCE-ALLOCATION-FAILED = 4
             EDJE-LOAD-ERROR-CORRUPT-FILE = 5
             EDJE-LOAD-ERROR-UNKNOWN-FORMAT = 6
             EDJE-LOAD-ERROR-INCOMPATIBLE-FILE = 7)))
  
  (define-cstruct* _EdjeMessageString
    ((str _string)))
  (define-cstruct* _EdjeMessageInt
    ((val _sint)))
  (define-cstruct* _EdjeMessageFloat
    ((val _double)))
  (define-cstruct* _EdjeMessageStringSet
    ((count _sint) (str _string))) ;FIX
  (define-cstruct* _EdjeMessageIntSet
    ((count _sint) (val _sint))) ;FIX
  (define-cstruct* _EdjeMessageFloatSet
    ((count _sint) (val _double))) ;FIX
  (define-cstruct* _EdjeMessageStringInt
    ((str _string) (val _sint)))
  (define-cstruct* _EdjeMessageStringFloat
    ((str _string) (val _double)))
  (define-cstruct* _EdjeMessageStringIntSet
    ((str _string) (count _sint) (val _sint))) ;FIX
  (define-cstruct* _EdjeMessageStringFloatSet
    ((str _string) (count _sint) (val _double))) ;FIX
  
  ;;Functions
  (defedje* edje-init : -> _sint)
  (defedje* edje-shutdown : -> _sint)
  
  (defedje* edje-frametime-set : _double -> _void)
  (defedje* edje-frametime-get : -> _double)
  
  (defedje* edje-freeze : -> _void)
  (defedje* edje-thaw : -> _void)
  (defedje* edje-fontset-append-set : _string -> _void)
  (defedje* edje-fontset-append-get : -> _string)
  
  (defedje* edje-file-collection-list :
    (post: (register-finalizer o_file-collection
                               edje-file-collection-list-free))
    _file
    -> (o_file-collection : (_EvasList _file)))
  (defedje edje-file-collection-list-free : _pointer -> _void)
  (defedje* edje-file-cache-set : _sint -> _void)
  (defedje* edje-file-cache-get : -> _sint)
  (defedje* edje-file-cache-flush : -> _void)
  (defedje* edje-collection-cache-set : _sint -> _void)
  (defedje* edje-collection-cache-get : -> _sint)
  (defedje* edje-collection-cache-flush : -> _void)
  
  (defedje* edje-color-class-set :
    _string
    _sint _sint _sint _sint
    _sint _sint _sint _sint
    _sint _sint _sint _sint
    -> _void)
  (defedje* edje-text-class-set : _string _string _EvasFontSize -> _void)
  (defedje* edje-extern-object-min-size-set :
    _EvasObject _EvasCoord _EvasCoord
    -> _void)
  (defedje* edje-extern-object-max-size-set :
    _EvasObject _EvasCoord _EvasCoord
    -> _void)
  
  (defedje* edje-object-add : _Evas -> _EdjeObject)
  
  (defedje* edje-object-data-get : _EvasObject _string -> _string)
  
  (defedje* edje-object-file-set : _EdjeObject _file _string -> _bool)
  (defedje* edje-object-file-get :
    _EdjeObject
    (o_file : (_ptr o _file))
    (o_part : (_ptr o _string))
    -> _void
    -> (list o_file o_part))
  (defedje* edje-object-load-error-get : _EdjeObject -> _EdjeLoadError)
  
  (defedje* edje-object-signal-callback-add :
    _EdjeObject _string _string
    (_fun _scheme _EdjeObject _string _string -> _cbvoid)
    _scheme
    -> _void)
  (defedje* edje-object-signal-callback-del :
    _EdjeObject _string _string
    (_fun _scheme _EdjeObject _string _string -> _cbvoid)
    -> _scheme)
  (defedje* edje-object-signal-emit : _EdjeObject _string _string -> _void)
  (defedje* edje-object-play-set : _EdjeObject _bool -> _void)
  (defedje* edje-object-play-get : _EdjeObject -> _bool)
  (defedje* edje-object-animation-set : _EdjeObject _bool -> _void)
  (defedje* edje-object-animation-get : _EdjeObject -> _bool)

  (defedje* edje-object-freeze : _EdjeObject -> _sint)
  (defedje* edje-object-thaw : _EdjeObject -> _sint)
  
  (defedje* edje-object-color-class-set :
    _EdjeObject _string
    _sint _sint _sint _sint
    _sint _sint _sint _sint
    _sint _sint _sint _sint
    -> _void)
  
  (defedje* edje-object-text-class-set :
    _EdjeObject _string _string _EvasFontSize
    -> _void)
  
  (defedje* edje-object-size-min-get :
    _EdjeObject
    (o_minw : (_ptr o _EvasCoord))
    (o_minh : (_ptr o _EvasCoord))
    -> _void
    -> (list o_minw o_minh))

  (defedje* edje-object-size-max-get :
    _EdjeObject
    (o_maxw : (_ptr o _EvasCoord))
    (o_maxh : (_ptr o _EvasCoord))
    -> _void
    -> (list o_maxw o_maxh))
  
  (defedje* edje-object-calc-force : _EdjeObject -> _void)
  
  (defedje* edje-object-size-min-calc :
    _EdjeObject
    (o_minw : (_ptr o _EvasCoord))
    (o_minh : (_ptr o _EvasCoord))
    -> _void
    -> (list o_minw o_minh))
  
  (defedje* edje-object-part-exists? : _EdjeObject _string -> _bool)
  
  (defedje* edje-object-part-geometry-get :
    _EdjeObject _string
    (o_x : (_ptr o _EvasCoord))
    (o_y : (_ptr o _EvasCoord))
    (o_w : (_ptr o _EvasCoord))
    (o_h : (_ptr o _EvasCoord))
    -> _void
    -> (list o_x o_y o_w o_h))
  
  (defedje* edje-object-text-change-cb-set :
    _EdjeObject
    (_fun _scheme _EdjeObject _string -> _cbvoid)
    _scheme
    -> _void)
  
  (defedje* edje-object-part-text-set : _EdjeObject _string _string -> _void)
  (defedje* edje-object-part-text-get :
    _EdjeObject _string
    -> _string)
  
  (defedje* edje-object-part-swallow : _EdjeObject _string _EvasObject -> _void)
  (defedje* edje-object-part-unswallow : _EdjeObject _EvasObject -> _void)
  (defedje* edje-object-part-swallow-get :
    _EdjeObject _string
    -> _EvasObjectAuto)
  
  (defedje* edje-object-part-state-get :
    _EdjeObject _string (o_ret : (_ptr o _double))
    -> (o_state : _string)
    -> (list o_state o_ret))
  
  (defedje* edje-object-part-drag-dir-get : _EdjeObject _string -> _sint)
  
  (defedje* edje-object-part-drag-value-set :
    _EdjeObject _string _double _double
    -> _void)
  (defedje* edje-object-part-drag-value-get :
    _EdjeObject _string (o_dx : (_ptr o _double)) (o_dy : (_ptr o _double))
    -> _void
    -> (list o_dx o_dy))

  (defedje* edje-object-part-drag-size-set :
    _EdjeObject _string _double _double
    -> _void)
  (defedje* edje-object-part-drag-size-get :
    _EdjeObject _string (o_dx : (_ptr o _double)) (o_dy : (_ptr o _double))
    -> _void
    -> (list o_dx o_dy))
  
  (defedje* edje-object-part-drag-step-set :
    _EdjeObject _string _double _double
    -> _void)
  (defedje* edje-object-part-drag-step-get :
    _EdjeObject _string (o_dx : (_ptr o _double)) (o_dy : (_ptr o _double))
    -> _void
    -> (list o_dx o_dy))
  
  (defedje* edje-object-part-drag-page-set :
    _EdjeObject _string _double _double
    -> _void)
  (defedje* edje-object-part-drag-page-get :
    _EdjeObject _string (o_dx : (_ptr o _double)) (o_dy : (_ptr o _double))
    -> _void
    -> (list o_dx o_dy))
  
  (defedje* edje-object-part-drag-step :
    _EdjeObject _string _double _double
    -> _void)
  (defedje* edje-object-part-drag-page :
    _EdjeObject _string _double _double
    -> _void)
  
  (defedje* edje-object-message-send :
    _EdjeObject _EdjeMessageType _sint _pointer
    -> _void) ;FIX
  
  (define (message-type->cstruct type)
    (case type
      ((EDJE-MESSAGE-NONE) _pointer)
      ((EDJE-MESSAGE-STRING) _EdjeMessageString-pointer)
      ((EDJE-MESSAGE-INT) _EdjeMessageInt-pointer)
      ((EDJE-MESSAGE-FLOAT) _EdjeMessageFloat-pointer)
      ((EDJE-MESSAGE-STRING-SET) _EdjeMessageStringSet-pointer)
      ((EDJE-MESSAGE-INT-SET) _EdjeMessageIntSet-pointer)
      ((EDJE-MESSAGE-FLOAT-SET) _EdjeMessageFloatSet-pointer)
      ((EDJE-MESSAGE-STRING-INT) _EdjeMessageStringInt-pointer)
      ((EDJE-MESSAGE-STRING-FLOAT) _EdjeMessageStringFloat-pointer)
      ((EDJE-MESSAGE-STRING-INT-SET) _EdjeMessageStringIntSet-pointer)
      ((EDJE-MESSAGE-STRING-FLOAT-SET) _EdjeMessageStringFloatSet-pointer)
      (else _pointer)))
  
  (provide edje-object-message-handler-set)
  (define (edje-object-message-handler-set o fun data)
    (let ((wfun (lambda (o type id msg)
                  (fun o type id
                       (cast-to-type msg
                                     (message-type->cstruct type))))))
      ((get-ffi-obj "edje_object_message_handler_set" libedje
                   (_fun _EdjeObject
                         (_fun _scheme _EdjeObject _EdjeMessageType _sint
                               _pointer -> _cbvoid)
                         _scheme
                         -> _void)) o wfun data)))
      
#|  (defedje* edje-object-message-handler-set :
    _EdjeObject
    (_fun _scheme _EdjeObject _EdjeMessageType _sint _pointer -> _cbvoid)
    _scheme
    -> _void)|#
  
  (defedje* edje-object-message-signal-process : _EdjeObject -> _void)
  (defedje* edje-message-signal-process : -> _void))
