#!/bin/sh
#|
exec mzscheme -qr "$0" ${1+"$@"}
|#

(require "eet.ss"
         (lib "cmdline.ss")
         (lib "file.ss"))

(define (depack-file ef file)
  (with-handlers ([eet-error? (lambda (exn)
                                (display (exn-message exn))
                                (newline))])
    (let ((data (eet-read ef file)))
      (unless (or (char=? (string-ref file 0) #\/)
                  (string=? file "..")
                  (string=? file "../")
                  (regexp-match "/\\.\\./" file)
                  (and (>= (string-length file) 3)
                       (regexp-match "/\\.\\.$" file)))
        (let ((dir (regexp-match ".*/" file)))
          (when dir (make-directory* (car dir)))))
      (call-with-output-file file
        (lambda (f)
          (write-bytes data f)
          (printf "exported ~a\n" file))))))

(define (depack pak-file)
  (with-handlers ([eet-error? (lambda (exn)
                                (display (exn-message exn))
                                (newline))])
    (call-with-eet-file pak-file 'EET-FILE-MODE-READ
      (lambda (ef)
        (for-each (lambda (n) (depack-file ef n)) (eet-list ef "*"))))))

(define (listit pak-file)
  (with-handlers ([eet-error? (lambda (exn)
                                (display (exn-message exn))
                                (newline))])
    (call-with-eet-file pak-file 'EET-FILE-MODE-READ
      (lambda (ef)
        (for-each (lambda (n) (printf "~a\n" n)) (eet-list ef "*"))))))

(define (pak-file ef file noz)
  (when (member 'read (file-or-directory-permissions file))
    (call-with-input-file file
      (lambda (f)
        (let ((buf (read-bytes (file-size file) f))
              (compress (let loop ((noz noz))
			  (if (null? noz)
			      #t
			      (if (regexp-match (car noz) file)
				  #f
				  (loop (cdr noz)))))))
          (eet-write ef file buf compress)
          (if compress
              (printf "compress: ~a\n" file)
              (printf "imported: ~a\n" file)))))))

(define (pak-dir ef dir noz)
    (if (directory-exists? dir)
        (for-each (lambda (d)
                    (pak-dir ef (build-path dir d) noz))
                  (directory-list dir))
        (pak-file ef dir noz)))

(define (pack pak-file files noz)
  (with-handlers ([eet-error? (lambda (exn)
                                (display (exn-message exn))
                                (newline))])
    (call-with-eet-file pak-file 'EET-FILE-MODE-WRITE
      (lambda (ef)
        (for-each (lambda (file) (pak-dir ef file noz)) files)
        (display "done.\n")))))

(define (main)
  (define help
    (string-append
     "usage:\n"
     "  eet -l in_file\n"
     "  eet -d in_file\n"
     "  eet -c out_file [-nz glob [-nz glob ...]] dir_file1 [dir_file2 ...]\n"
     "\n"
     "where:\n"
     "  -l in_file     list contents of eet file\n"
     "  -d in_file     unpack eet file\n"
     "  -c out_file    pack up eet file\n"
     "  -nz match      don't compress files matching match glob\n"
     "\n"
     "example:\n"
     "  eet -c out.eet -nz \"*.jpg\" things/\n"
     "  eet -l out.eet\n"
     "  eet -d out.eet\n"))
 
  (cond
    ((= (vector-length argv) 2)
     (cond
       ((string=? (vector-ref argv 0) "-d") (depack (vector-ref argv 1))
                                            (exit 0))
       ((string=? (vector-ref argv 0) "-l") (listit (vector-ref argv 1))
                                            (exit 0))
       (else (display help) -1)))
    ((and (> (vector-length argv) 2)
          (string=? (vector-ref argv 0) "-c"))
     (let ((noz ())
           (files ()))
       (do ((i 2 (if (string=? (vector-ref argv i) "-nz")
                     (+ i 2)
                     (add1 i)))
            (max (vector-length argv)))
         ((>= i max))
         (if (string=? (vector-ref argv i) "-nz")
             (unless (>= i (sub1 max))
               (set! noz (cons (vector-ref argv (add1 i)) noz)))
             (set! files (cons (vector-ref argv i) files))))
       (pack (vector-ref argv 1) files noz)
       (exit 0)))
    (else (display help)
          (exit -1))))

(main)
