(module eet mzscheme
  
  (require (lib "foreign.ss") 
           (lib "string.ss")
           (lib "serialize.ss")
           "efl-common.ss")
  
  (define libeet (ffi-lib "libeet"))
  
  (provide (struct eet-error ()) (struct eet-error:I/O ()))
  (define-struct (eet-error efl-error) ())
  (define-struct (eet-error:I/O eet-error) ())

  (define-efl-exn-raiser eet)
  (define-efl-ffi-definer eet)
  
  (define-cpointer-type _EetFile)

  (define _EetFileMode
    (_enum '(EET-FILE-MODE-INVALID = -1
             EET-FILE-MODE-READ
             EET-FILE-MODE-WRITE
             EET-FILE-MODE-READ-WRITE)))
  
  ;; Functions
  (defeet eet-init : -> _sint)
  (defeet eet-shutdown : -> _sint)

  (defeet* eet-open : (pre: (eet-init))
    (post: (unless o_eet-file
             (eet-raise-exn I/O : "Failed to open ~a. ~a" i_file
                            (case i_mode
                              [(EET-FILE-MODE-READ) "File not found."]
                              [(EET-FILE-MODE-READ-WRITE
                                EET-FILE-MODE-WRITE) "Permission denied."]
                              [else "Invalid mode."]))))
    (i_file : _file) (i_mode : _EetFileMode)
    -> (o_eet-file : _EetFile/null))
  
  (defeet* eet-mode-get : _EetFile -> _EetFileMode)
  
  (defeet* eet-close : (post: (eet-shutdown)) _EetFile -> _void)
  
  (defeet* eet-read :
    (pre: (when (eq? (eet-mode-get i_eet-file) 'EET-FILE-MODE-WRITE)
            (error "Cannot read, not in read mode."))
     post: (if o_data
               (register-finalizer o_data free)
               (eet-raise-exn I/O : "Failed to read data from '~a'." i_key)))
    (i_eet-file : _EetFile) (i_key : _string) (o_size : (_ptr o _sint))
    -> (o_data : (_byte-vector o_size)))
  
  (defeet* eet-write :
    (pre: (when (eq? (eet-mode-get i_eet-file) 'EET-FILE-MODE-READ)
            (error "Cannot write, not in write mode."))
     post: (when (= 0 o_success)
             (eet-raise-exn I/O : "Failed to write data to '~a'." i_key)))
    (i_eet-file : _EetFile) (i_key : _string)
    (i_data : _bytes) (_sint = (bytes-length i_data)) _bool
    -> (o_success : _sint))
    
  (defeet* eet-deserialize "eet_read" :
    (pre: (when (eq? (eet-mode-get i_eet-file) 'EET-FILE-MODE-WRITE)
            (eet-raise-exn I/O : "Cannot read, not in read mode."))
     post: (unless o_data
             (eet-raise-exn I/O : "Failed to read data from '~a'." i_key)))
    (i_eet-file : _EetFile) (i_key : _string) (_ptr o _sint)
    -> (o_data : _string/utf-8/free)
    -> (deserialize (read-from-string o_data)))

  (defeet* eet-serialize "eet_write" : (eet-file key data compress) ::
    (pre: (when (eq? (eet-mode-get eet-file) 'EET-FILE-MODE-READ)
            (eet-raise-exn I/O : "Cannot write, not in write mode."))
     post: (unless o_success
             (eet-raise-exn I/O : "Failed to write data to '~a'." key)))
    (_EetFile = eet-file)
    (_string = key)
    (i_data : _string = (expr->string (serialize data)))
    (_sint = (add1 (string-length i_data)))
    (_bool = compress)
    -> (o_success : _bool))
  
  (defeet* eet-delete :
    (pre: (when (eq? (eet-mode-get i_eet-file) 'EET-FILE-MODE-READ)
            (eet-raise-exn I/O : "Cannot delete, not in write mode."))
     post: (unless o_success
             (eet-raise-exn I/O : "Failed to delete '~a'." i_key)))
    (i_eet-file : _EetFile) (i_key : _string)
    -> (o_success : _bool))
  
  (defeet* eet-list :
    (pre: (when (eq? (eet-mode-get i_eet-file) 'EET-FILE-MODE-WRITE)
            (error "Cannot list entries, not in read mode.")))
    (i_eet-file : _EetFile) _string (o_count : (_ptr o _sint))
    -> (_list/free _string o_count))
  
  (defeet* eet-data-image-header-read :
    (pre: (when (eq? (eet-mode-get i_eet-file) 'EET-FILE-MODE-WRITE)
            (error "Cannot read, not in read mode."))
     post: (when (= 0 o_success)
             (eet-raise-exn I/O : "Failed to read image header from '~a'."
                                  i_key)))
    (i_eet-file : _EetFile) (i_key : _string)
    (o_w : (_ptr o _sint))
    (o_h : (_ptr o _sint))
    (o_alpha : (_ptr o _sint))
    (o_compress : (_ptr o _bool))
    (o_quality : (_ptr o _sint))
    (o_lossy : (_ptr o _sint))
    -> (o_success : _sint)
    -> (list o_w o_h o_alpha o_compress o_quality o_lossy))
  
  (defeet* eet-data-image-read :
    (pre: (when (eq? (eet-mode-get i_eet-file) 'EET-FILE-MODE-WRITE)
              (error "Cannot read, not in read mode."))
     post: (if o_data
               (register-finalizer o_data free)
               (eet-raise-exn I/O : "Failed to read image data from '~a'."
                                    i_key)))
    (i_eet-file : _EetFile) (i_key : _string)
    (o_w : (_ptr o _sint))
    (o_h : (_ptr o _sint))
    (o_alpha : (_ptr o _sint))
    (o_compress : (_ptr o _bool))
    (o_quality : (_ptr o _sint))
    (o_lossy : (_ptr o _sint))
    -> (o_data : (_byte-vector (* o_w o_h)))
    -> (list o_data o_w o_h o_alpha o_compress o_quality o_lossy))
  
  (defeet* eet-data-image-write :
    (pre: (begin
            (when (eq? (eet-mode-get i_eet-file) 'EET-FILE-MODE-READ)
              (error "Cannot write, not in write mode."))
            (when (< (bytes-length i_data) (* i_w i_h))
              (error "The amount of bytes is less than w * h.")))
     post: (when (= 0 o_success)
             (eet-raise-exn I/O : "Failed to write image data to '~a'."
                                  i_key)))
    (i_eet-file : _EetFile) (i_key : _string)
    (i_data : _bytes) (i_w : _sint) (i_h : _sint) _sint _sint _sint _sint
    -> (o_success : _sint))
  
  (defeet* eet-data-image-header-decode :
    (post: (when (= 0 o_success)
             (eet-raise-exn : "Failed to decode image header.")))
    (i_data : _bytes) (_sint = (bytes-length i_data))
    (o_w : (_ptr o _sint))
    (o_h : (_ptr o _sint))
    (o_alpha : (_ptr o _sint))
    (o_compress : (_ptr o _bool))
    (o_quality : (_ptr o _sint))
    (o_lossy : (_ptr o _sint))
    -> (o_success : _sint)
    -> (list o_w o_h o_alpha o_compress o_quality o_lossy))
  
  (defeet* eet-data-image-decode :
    (post: (if o_data
               (register-finalizer o_data free)
               (eet-raise-exn : "Failed to decode image data.")))
    (i_data : _bytes) (_sint  = (bytes-length i_data))
    (o_w : (_ptr o _sint))
    (o_h : (_ptr o _sint))
    (o_alpha : (_ptr o _sint))
    (o_compress : (_ptr o _bool))
    (o_quality : (_ptr o _sint))
    (o_lossy : (_ptr o _sint))
    -> (o_data : (_byte-vector (* o_w o_h)))
    -> (list o_data o_w o_h o_alpha o_compress o_quality o_lossy))
  
  (defeet* eet-data-image-encode :
    (post: (if o_data
               (register-finalizer o_data free)
               (eet-raise-exn : "Failed to encode image data.")))
    _bytes (o_size : (_ptr o _sint)) _sint
    _sint _sint _sint _sint _sint
    -> (o_data : (_byte-vector o_size)))
  
  (provide call-with-eet-file)
  (define (call-with-eet-file filename mode proc)
    (define eet-file #f)
    (dynamic-wind
     (lambda () (set! eet-file (eet-open filename mode)))
     (lambda () (proc eet-file))
     (lambda () (eet-close eet-file)))))