;; TODO: add exceptions where needed
;; TODO: play well with unsafe! and provide-unsafe
;; TODO: a new type for the messages with variable size on edje
;; TODO: unions, structs/vectors inside structs (custimized define-cstruct)
;; TODO: a new (ecore-main-loop-begin) that doesn't steal control from mzscheme?? (using ecore-main-loop-iterate)
(module efl-common mzscheme
  
  (require (lib "foreign.ss"))
  (unsafe!)

  (provide (struct efl-error ()))
  (define-struct (efl-error exn:fail) ())
  
  (provide define-efl-exn-raiser)
  (define-syntax (define-efl-exn-raiser stx)
    (define (make-syntax NAME-stx)
      (define name (symbol->string (syntax-e NAME-stx)))
      (define (id . strings)
        (datum->syntax-object
         NAME-stx (string->symbol (apply string-append strings)) NAME-stx))
      (with-syntax ([NAME-raise-exn  (id name"-raise-exn")]
                    [NAME            name])
        #'(define-syntax (NAME-raise-exn stx)
            (syntax-case stx (:)
               [(_ type : message a1 (... ...))
                (let ((exn-ctor #'(string->symbol 
                                   (string-append "make-"NAME"-error:"
                                                  (symbol->string 'type)))))
                  #`(raise ((eval #,exn-ctor)
                            (string->immutable-string
                             (format message a1 (... ...)))
                            (current-continuation-marks))))]
               [(_ : message a1 (... ...))
                (let ((exn-ctor #'(string->symbol 
                                   (string-append "make-"NAME"-error"))))
                  #`(raise ((eval #,exn-ctor)
                            (string->immutable-string
                             (format message a1 (... ...)))
                            (current-continuation-marks))))]))))
    (syntax-case stx ()
      [(_ lib) (make-syntax #'lib)]))
  
  (provide define-efl-ffi-definer)
  (define-syntax (define-efl-ffi-definer stx)
    (define (make-syntax NAME-stx)
      (define name (symbol->string (syntax-e NAME-stx)))
      (define (id . strings)
        (datum->syntax-object
         NAME-stx (string->symbol (apply string-append strings)) NAME-stx))
      (with-syntax ([defNAME  (id "def"name)]
                    [defNAME* (id "def"name"*")]
                    [libNAME  (id "lib"name)])
        #'(define-syntaxes (defNAME defNAME*)
            (values
             (lambda (nstx)
               (syntax-case nstx (:)
                 [(_ fname : x (... ...))
                  #'(define fname
                      (get-ffi-obj
                       (regexp-replaces
                        'fname '((#rx"-" "_") (#rx"[?!]$" "")))
                       libNAME (_fun x (... ...))))]
                 [(_ fname cname : x (... ...))
                  #'(define fname
                      (get-ffi-obj cname libNAME (_fun x (... ...))))]))
             (lambda (nstx)
               (syntax-case nstx (:)
                 [(_ fname : x (... ...))
                  #'(begin (provide fname) (defNAME fname : x (... ...)))]
                 [(_ fname cname : x (... ...))
                  #'(begin
                      (provide fname)
                      (defNAME fname cname : x (... ...)))]))))))
    (syntax-case stx ()
      [(_ lname) (make-syntax #'lname)]))
    
  (provide def-extern)
  (define-syntax def-extern
    (syntax-rules ()
      [(_ sym lib type)
       (begin
         (provide sym)
         (define sym (make-c-parameter
                      (regexp-replaces 'sym '((#rx"-" "_") (#rx"[?!]$" "")))
                      lib type)))]))

  (provide efl-raise-exn)
  (define-efl-exn-raiser efl)
  
  (provide _string/utf-8/free)
  (define-fun-syntax _string/utf-8/free
    (syntax-id-rules ()
      [_ (type: _bytes
          post: (x => (begin0
                        (bytes->string/utf-8 x)
                        (free x))))]))
  
  (provide _list/free)
  (define-fun-syntax _list/free
    (syntax-rules ()
      [(_ t c) (type: _pointer
                post: (x => (begin0
                              (cblock->list x t c)
                              (free x))))]))
  
  (provide _list/finalize)
  (define-fun-syntax _list/finalize
    (syntax-rules ()
      [(_ t c) (type: _pointer
                post: (x => (let ((ret (cblock->list x t c)))
                              (register-finalizer ret (lambda () (free x)))
                              ret)))]))
  
  (provide _byte-vector)
  (define-fun-syntax _byte-vector
    (syntax-rules ()
      [(_ n) (type: _bytes
              post: (x => (make-sized-byte-string x n)))]))
                    
  (provide _cbvoid)
  (define _cbvoid (make-ctype _byte (lambda (o) 0) (lambda (o) void)))
  
  (provide cast-to-type)
  (define (cast-to-type val type)
    (let ((ptr (malloc 1 _pointer)))
      (ptr-set! ptr _pointer val)
      (ptr-ref ptr type)))
  
  (provide provide-cstruct)
  (define-syntax (provide-cstruct stx)
    (define (make-syntax _TYPE-stx slot-names-stx)
      (define name
        (cadr (regexp-match #rx"^_(.+)$"
                            (symbol->string (syntax-e _TYPE-stx)))))
      (define slot-names (map (lambda (x) (symbol->string (syntax-e x)))
                              (syntax->list slot-names-stx)))
      (define (id . strings)
        (datum->syntax-object
         _TYPE-stx (string->symbol (apply string-append strings)) _TYPE-stx))
      (define (ids name-func)
        (map (lambda (s stx)
               (datum->syntax-object
                stx (string->symbol (apply string-append (name-func s))) stx))
             slot-names (syntax->list slot-names-stx)))
      (with-syntax
          ([TYPE?                (id name"?")]
           [make-TYPE            (id "make-"name)]
           [list->TYPE           (id "list->"name)]
           [list*->TYPE          (id "list*->"name)]
           [TYPE->list           (id name"->list")]
           [TYPE->list*          (id name"->list*")]
           [TYPE-tag             (id name"-tag")]
           [(TYPE-SLOT ...)      (ids (lambda (s) `(,name"-",s)))]
           [(set-TYPE-SLOT! ...) (ids (lambda (s) `("set-",name"-",s"!")))])
        #'(provide TYPE? make-TYPE list->TYPE list*->TYPE TYPE->list TYPE->list*
                   TYPE-SLOT ... set-TYPE-SLOT! ...)))
    (syntax-case stx ()
      [(_ (cs super) (slot ...)) (make-syntax #'cs #'(slot ...))]
      [(_ cs (slot ...)) (make-syntax #'cs #'(slot ...))]))
  
  (provide define-cstruct*)
  (define-syntax (define-cstruct* stx)
    (syntax-case stx ()
      [(_ _TYPE ([slot slot-type] ...))
       #'(begin (provide-cstruct _TYPE (slot ...))
                (define-cstruct _TYPE ([slot slot-type] ...)))]
      [(_ (_TYPE _SUPER) ([slot slot-type] ...))
       #'(begin (provide-cstruct _TYPE (slot ...))
                (define-cstruct (_TYPE _SUPER) ([slot slot-type] ...)))]))

  (define (max-sized-type types)
    (let loop ((maxt (car types)) (types (cdr types)))
      (if (null? types)
          maxt
          (let ((maxt (if (> (ctype-sizeof (car types))
                             (ctype-sizeof maxt))
                          (car types)
                          maxt)))
            (loop maxt (cdr types))))))
  
  (provide define-cunion)
  (define-syntax (define-cunion stx) ;;FIXME bloque del tamaño incorrecto (algo para obtener el elemento más grande)
    (define (make-syntax _TYPE-stx slot-names-stx slot-types-stx)
      (define name
        (cadr (regexp-match #rx"^_(.+)$"
                            (symbol->string (syntax-e _TYPE-stx)))))
      (define slot-names (map (lambda (x) (symbol->string (syntax-e x)))
                              (syntax->list slot-names-stx)))
      (define (id . strings)
        (datum->syntax-object
         _TYPE-stx (string->symbol (apply string-append strings)) _TYPE-stx))
      (define (ids name-func)
        (map (lambda (s stx)
               (datum->syntax-object
                stx (string->symbol (apply string-append (name-func s))) stx))
             slot-names (syntax->list slot-names-stx)))
      (with-syntax
          ([first-type           (let ([xs (syntax->list slot-types-stx)])
                                   (and (pair? xs) (car xs)))]
           [name-string          name]
           [union-string        (format "union:~a" name)]
           [(slot ...)           slot-names-stx]
           [(slot-type ...)      slot-types-stx]
           [_TYPE                _TYPE-stx]
           [_TYPE-pointer        (id "_"name"-pointer")]
           [_TYPE*               (id "_"name"*")]
           [TYPE?                (id name"?")]
           [make-TYPE            (id "make-"name)]
           [TYPE-tag             (id name"-tag")]
           [(stype ...)          (ids (lambda (s) `(,name"-",s"-type")))]
           [(TYPE-SLOT ...)      (ids (lambda (s) `(,name"-",s)))]
           [(set-TYPE-SLOT! ...) (ids (lambda (s) `("set-",name"-",s"!")))])
        #'(define-values (_TYPE _TYPE-pointer TYPE? TYPE-tag make-TYPE
                                TYPE-SLOT ... set-TYPE-SLOT! ...)
            (let*-values ([(stype ...)  (values slot-type ...)]
                          [(types)      (list stype ...)])
              (define-cpointer-type _TYPE)
              (define _TYPE* (_cpointer TYPE-tag
                                        (make-cstruct-type
                                         (list (max-sized-type types)))))
              (define all-tags (cons TYPE-tag ()))
              (define all-types types)
              (define (TYPE-SLOT x)
                (unless (TYPE? x) (raise-type-error 'TYPE-SLOT union-string x))
                (ptr-ref x stype))
              ...
              (define (set-TYPE-SLOT! x slot)
                (unless (TYPE? x)
                  (raise-type-error 'set-TYPE-SLOT! union-string 0 x slot))
                (ptr-set! x stype slot))
              ...
              (define make-TYPE
                (lambda ()
                  (let ([block (malloc _TYPE*)])
                    (set-cpointer-tag! block all-tags)
                    block)))
              (values _TYPE* _TYPE TYPE? TYPE-tag make-TYPE
                      TYPE-SLOT ... set-TYPE-SLOT! ...)))))
    (define (identifiers? stx)
      (andmap identifier? (syntax->list stx)))
    (define (_-identifier? stx)
      (and (identifier? stx)
           (regexp-match #rx"^_.+" (symbol->string (syntax-e stx)))))
    (syntax-case stx ()
      [(_ _TYPE ([slot slot-type] ...))
       (and (_-identifier? #'_TYPE) (identifiers? #'(slot ...)))
       (make-syntax #'_TYPE #'(slot ...) #'(slot-type ...))])))