(module evas-types mzscheme
  
  (require (lib "foreign.ss")
           "efl-common.ss")
  
  (provide _Evas _Evas/null _EvasObject _EvasObjectAuto _EvasObjectGradient
           _EvasObjectImage _EvasObjectIntercept _EvasObjectLine
           _EvasObjectPolygon _EvasObjectRectangle _EvasObjectSmart
           _EvasObjectTextblock _EvasObjectText _EdjeObject _EvasModifier
           _EvasLock _EvasList _EvasFontSize _EvasModifierMask _EvasCoord
           _EvasAngle _EvasBool _EvasSmart)

  (define libevas (ffi-lib "libevas"))
  
  (define-efl-ffi-definer evas)

  ;; Finds out the type of the object and returns it
  (define (EvasObjectAuto->Scheme eo)
    (defevas evas-object-type-get : _pointer -> _symbol)
    
    (case (evas-object-type-get eo)
      [(edje) (cast-to-type eo _EdjeObject)]
      [(gradient) (cast-to-type eo _EvasObjectGradient)]
      [(image) (cast-to-type eo _EvasObjectImage)]
      [(line) (cast-to-type eo _EvasObjectLine)]
      [(polygon) (cast-to-type eo _EvasObjectPolygon)]
      [(rectangle) (cast-to-type eo _EvasObjectRectangle)]
      [(smart) (cast-to-type eo _EvasObjectSmart)]
      [(textblock) (cast-to-type eo _EvasObjectTextblock)]
      [(text) (cast-to-type eo _EvasObjectText)]))
  
  (define _EvasFontSize _sint)
  (define _EvasModifierMask _ulong)
  (define _EvasCoord _sint)
  (define _EvasAngle _sint)
  (define _EvasBool (make-ctype _uint8
                                (lambda (b) (if b 1 0))
                                (lambda (n) (if (= 0 n) #f #t))))
  
  (define-cpointer-type _Evas)
  (define-cpointer-type _EvasObject)
  (define-cpointer-type _EvasObjectAuto _EvasObject #f EvasObjectAuto->Scheme)
  (define-cpointer-type _EdjeObject _EvasObject)
  (define-cpointer-type _EvasObjectGradient _EvasObject)
  (define-cpointer-type _EvasObjectImage _EvasObject)
  (define-cpointer-type _EvasObjectIntercept _EvasObject)
  (define-cpointer-type _EvasObjectLine _EvasObject)
  (define-cpointer-type _EvasObjectPolygon _EvasObject)
  (define-cpointer-type _EvasObjectRectangle _EvasObject)
  (define-cpointer-type _EvasObjectSmart _EvasObject)
  (define-cpointer-type _EvasObjectTextblock _EvasObject)
  (define-cpointer-type _EvasObjectText _EvasObject)
  (define-cpointer-type _EvasModifier)
  (define-cpointer-type _EvasLock)
  (define-cpointer-type _EvasSmart)

  ;; Convert a list to an EvasList
  (define (list->EvasList lst type)
    (defevas e-list-append : _pointer type -> _pointer)

    (let loop ((elst #f) (lst lst))
      (if (null? lst)
          elst
          (loop (e-list-append elst (car lst)) (cdr lst)))))
  
  ;; Convert an EvasList to a list
  (define (EvasList->list elst type)
    (defevas evas-list-next : _pointer -> _pointer)
    (defevas evas-list-data : _pointer -> type)
    (defevas evas-list-free : _pointer -> _pointer)
    
    (begin0
      (let loop ((lst '()) (elst elst))
        (if (not elst)
            lst
            (loop (cons (evas-list-data elst) lst) (evas-list-next elst))))
      (evas-list-free elst)))
  
  (define-fun-syntax _EvasList
    (syntax-rules (i o io)
      [(_ i  t) (type: _pointer
                 pre: (x => (list->EvasList x t)))]
      [(_ o  t) (type: _pointer
                 pre: #f
                 post: (x => (EvasList->list x t)))]
      [(_ io t) (type: _pointer
                 pre: (x => (list->EvasList x t)))]
      [(_    t) (type: _pointer
                 post: (x => (EvasList->list x t)))])))
